package com.macro.mall.tiny.common.util;


import com.google.common.collect.Lists;
import com.macro.mall.tiny.config.UploadConfig;
import com.macro.mall.tiny.domain.AdminUserDetails;
import com.macro.mall.tiny.modules.business.model.BDirectory;
import com.macro.mall.tiny.modules.business.model.BFile;
import com.macro.mall.tiny.modules.business.model.BFileDetail;
import com.macro.mall.tiny.security.util.AuthUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * 文件操作工具类
 */
public class MyFileUtils {

    /**
     * 写入文件
     *
     * @param target
     * @param src
     * @throws IOException
     */
    public static void write(String target, InputStream src) throws IOException {
        OutputStream os = new FileOutputStream(target);
        byte[] buf = new byte[1024];
        int len;
        while (-1 != (len = src.read(buf))) {
            os.write(buf, 0, len);
        }
        os.flush();
        os.close();
    }

    /**
     * 分块写入文件
     *
     * @param target
     * @param targetSize
     * @param src
     * @param srcSize
     * @param chunks
     * @param chunk
     * @throws IOException
     */
    public static void writeWithBlock(String target, Long targetSize, InputStream src, Long srcSize, Integer chunks, Integer chunk) throws IOException {
        RandomAccessFile randomAccessFile = new RandomAccessFile(target, "rw");
        randomAccessFile.setLength(targetSize);
        if (chunk == chunks - 1 && chunk != 0) {
            randomAccessFile.seek(chunk * (targetSize - srcSize) / chunk);
        } else {
            randomAccessFile.seek(chunk * srcSize);
        }
        byte[] buf = new byte[1024];
        int len;
        while (-1 != (len = src.read(buf))) {
            randomAccessFile.write(buf, 0, len);
        }
        randomAccessFile.close();
    }

    /**
     * 生成随机文件名
     *
     * @return
     */
    public static String generateFileName() {
        return UUID.randomUUID().toString();
    }

    /**
     * 获取文件扩展名
     *
     * @param file
     * @return
     */
    public static String getExt(MultipartFile file) {
        String fileName = file.getOriginalFilename();
        return fileName.substring(fileName.lastIndexOf(".") + 1);
    }

    public static String getAuthFilePath(String basePath, String fileName, boolean auth) {
        if (StringUtils.isBlank(basePath) || StringUtils.isBlank(fileName)) return "";
        String resultPath = basePath + UploadConfig.SEPARATOR + fileName;
        File file = new File(resultPath);
        if (file.exists()) {
            return resultPath;
        }
        resultPath = basePath + UploadConfig.SEPARATOR + UploadConfig.AUTH_STR + fileName;
        file = new File(resultPath);
        if (auth && file.exists()) {
            return resultPath;
        }
        return "";
    }

    public static List<BFile> getSonFiles(String basePath, AdminUserDetails userDetails) {
        boolean access = AuthUtil.checkAccess(userDetails, AuthUtil.ACCESS);
        return getSonFiles(basePath, null, access);
    }

    public static List<BFile> getSonFiles(String basePath, boolean auth) {
        return getSonFiles(basePath, null, auth);
    }

    public static List<BFile> getSonFiles(String basePath, FilenameFilter filter, boolean auth) {
        if (StringUtils.isBlank(basePath)) return Lists.newArrayList();
        File oriDic = new File(basePath);
        if (!oriDic.exists() || oriDic.isFile() || (!auth && oriDic.getName().startsWith(String.valueOf(UploadConfig.AUTH_STR)))) {
            return Lists.newArrayList();
        }
        if (oriDic.exists() && oriDic.isDirectory()) {
            File[] sonFiles = filter == null ? oriDic.listFiles() : oriDic.listFiles(filter);
            if (sonFiles == null) return Lists.newArrayList();
            return Arrays.stream(sonFiles)
                    .filter(son -> auth || !son.getName().startsWith("~"))
                    .map(son -> {
                        final BFile sonFile = new BFile();
                        sonFile.setName(son.getName());
                        sonFile.setPath(son.getPath());
                        sonFile.setIsDirectory(son.isDirectory() ? 1 : 0);
                        if (son.isDirectory()) {
                            sonFile.setSubFile(getSonFiles(son.getPath(), filter, auth));
                        }
                        return sonFile;
                    })
                    .collect(Collectors.toList());
        }
        return Lists.newArrayList();
    }

    /**
     * @param userDetails
     * @return com.macro.mall.tiny.modules.business.model.BDirectory
     * @describe 获取根目录下的所有文件夹
     * @author gjs
     * @date 2021/4/2
     * @since J7.23.0
     */
    public static BDirectory getRootDirectory(AdminUserDetails userDetails) {
        final boolean access = AuthUtil.checkAccess(userDetails, AuthUtil.ACCESS);
        final File rootDirectory = new File(UploadConfig.powerPath);
        return getAllAuthDirectory(rootDirectory, access);
    }

    public static BDirectory getAllAuthDirectory(File file, boolean access) {
        if (!file.exists() && !file.isDirectory()) {
            return null;
        }
        final String fileName = file.getName();
        List<BDirectory> subDirectory = null;
        boolean hasFile = false;

        final File[] subFiles = file.listFiles((dir, name) -> access || !name.startsWith(String.valueOf(UploadConfig.AUTH_STR)));
        if (subFiles != null && subFiles.length > 0) {
            for (File subFile : subFiles) {
                if (subFile.isFile()) {
                    hasFile = true;
                    break;
                }
            }
            subDirectory = Arrays.stream(subFiles)
                    .filter(subFile -> file.isDirectory())
                    .map(subFile -> getAllAuthDirectory(subFile, access))
                    .collect(Collectors.toList());
        }
        final BDirectory thisFile = new BDirectory();
        thisFile.setName(fileName);
        thisFile.setPath(file.getPath());
        thisFile.setSubDirectory(subDirectory == null ? Lists.newArrayList() : subDirectory);
        thisFile.setHasFile(hasFile ? 1 : 0);

        return thisFile;
    }

    public static List<BFileDetail> getFileDetailList(String path, AdminUserDetails userDetails) {
        if (StringUtils.isBlank(path)) return Lists.newArrayList();
        boolean access = AuthUtil.checkAccess(userDetails, AuthUtil.ACCESS);
        File oriDic = new File(path);
        if (!oriDic.exists() || oriDic.isFile() || (!access && oriDic.getName().startsWith(String.valueOf(UploadConfig.AUTH_STR)))) {
            return Lists.newArrayList();
        }
        File[] subFiles = oriDic.listFiles((dir, name) -> access || !name.startsWith(String.valueOf(UploadConfig.AUTH_STR)));
        if (subFiles == null) return Lists.newArrayList();
        return Arrays.stream(subFiles)
                .filter(File::isFile)
                .map(file -> {
                    BFileDetail fileDetail = new BFileDetail();
                    fileDetail.setName(file.getName());
                    fileDetail.setPath(file.getPath());
                    fileDetail.setUploadTime(new Date(file.lastModified()));
                    String fileName = file.getName();
                    int lastIndexOf = fileName.lastIndexOf(".");
                    fileDetail.setType(lastIndexOf == -1 ? null : fileName.substring(lastIndexOf + 1));
                    fileDetail.setSize(getPrettySize(file.length()));
                    return fileDetail;
                }).collect(Collectors.toList());
    }

    private static String getPrettySize(long size) {
        long rest;
        if (size < 1024) {
            return size + "B";
        } else {
            size /= 1024;
        }
        if (size < 1024) {
            return size + "KB";
        } else {
            rest = size % 1024;
            size /= 1024;
        }
        if (size < 1024) {
            size = size * 100;
            return size / 100 + "." + rest * 100 / 1024 % 100 + "MB";
        } else {
            size = size * 100 / 1024;
            return size / 100 + "." + size % 100 + "GB";
        }
    }
}
